.pybi-flow-box>.echart-box>.echart[data-v-cfe5a4f0]{min-width:30vw}.echart-box[data-v-cfe5a4f0]{position:relative;width:100%;height:100%}.echart-box .echart[data-v-cfe5a4f0]{min-height:30vh;width:100%;height:var(--06ba90f8)}.echart-box .debug-box[data-v-cfe5a4f0]{z-index:9999;position:absolute;top:0;left:0;transform:translateY(-100%);display:flex;max-width:100%}.echart-box .back[data-v-cfe5a4f0]{z-index:9999;display:inline-block;border:1px solid #000;border-radius:12%;padding:.2em}.echart-box .back[data-v-cfe5a4f0]:hover{cursor:pointer}
