:root{font-family:Inter,Avenir,Helvetica,Arial,sans-serif;font-size:16px}*{margin:0;padding:0;box-sizing:border-box}body{background-color:#f8fafc}.pybi-container{gap:.8em;display:flex;flex-direction:column;justify-content:flex-start;align-items:flex-start}.pybi-animate-bounce{animation:pybi-kf-bounce 1s infinite}.pybi-animate-pulse{animation:pybi-kf-pulse 2s cubic-bezier(.4,0,.6,1) infinite}.pybi-animate-ping{animation:pybi-kf-ping 1s cubic-bezier(0,0,.2,1) infinite}.pybi-animate-spin{animation:pybi-kf-spin 1s linear infinite}@keyframes pybi-kf-spin{0%{transform:rotate(0)}to{transform:rotate(360deg)}}@keyframes pybi-kf-ping{75%,to{transform:scale(2);opacity:0}}@keyframes pybi-kf-pulse{0%,to{opacity:1}50%{opacity:.5}}@keyframes pybi-kf-bounce{0%,to{transform:translateY(-25%);animation-timing-function:cubic-bezier(.8,0,1,1)}50%{transform:translateY(0);animation-timing-function:cubic-bezier(0,0,.2,1)}}.pybi-box[data-v-696d26b5]{position:relative;width:100%;display:flex;flex-direction:column;align-items:flex-start;justify-content:flex-start;overflow:auto;gap:.8em}.pybi-box .item-box[data-v-696d26b5]{width:100%;overflow:visible;display:flex}.pybi-flow-box[data-v-03d7ef6d]{display:flex;flex-direction:row;flex-wrap:wrap;align-items:flex-start;justify-content:var(--5f0fedb0);width:100%}.pybi-flow-box[data-v-03d7ef6d]>.pybi-space[data-auto-grow]{flex-grow:9999}.pybi-flow-box[data-v-03d7ef6d]>*[data-pybi-auto-width]{width:unset}.grid-box[data-v-8fd81f0c]{display:grid;grid-template-areas:var(--67400cc0);grid-template-columns:var(--0133259d);grid-template-rows:var(--be8267ce);gap:.8rem;justify-items:var(--29f6d910);align-items:var(--ac1c9e22);width:100%}.grid-box>.item[data-v-8fd81f0c]{display:flex}.grid-box>.item[data-v-8fd81f0c]>p{overflow:hidden;overflow-wrap:break-word}.grid-box[data-v-8fd81f0c]>*{overflow-x:auto;width:100%}.textValue-box[data-v-eb443eb6]{display:inline-block;white-space:pre-wrap;margin:0;box-sizing:initial}.md-box[data-v-6a82eb51]{width:100%;height:var(--59de2b90)}.pybi-svg-icon[data-v-3f2a90e4]{display:inline-flex;justify-content:center;align-items:center}.pybi-space[data-v-baf5cd60]{width:var(--38253cf2);height:var(--38253cf2);visibility:collapse}.err[data-v-938c3bc4]{width:100%;color:red;border:2px dashed #ba4949;padding:.8rem}.err>p[data-v-938c3bc4]:first-child{font-size:1.2rem;margin:.2rem 0}html[data-v-6f312624]{font-size:1.1115vw}.app-box[data-v-6f312624]{padding:.8rem 2rem;min-height:100vh}
