.ex-cards-zone-box[data-v-bd7eb388]{font-size:.5em;display:grid;grid-template-columns:repeat(auto-fit,minmax(20em,1fr));overflow:auto;max-height:40em;gap:.5em}.ex-cards-zone-box *[data-v-bd7eb388]{padding:0;margin:0}.ex-cards-zone-box .title[data-v-bd7eb388]{font-weight:700}.ex-cards-zone-box .card[data-v-bd7eb388]{background-color:#fff;box-shadow:0 1px 2px #0000000d;border-radius:.3em;width:auto;overflow:auto}.ex-cards-zone-box .card>.title[data-v-bd7eb388]{color:azure;background-color:#6c6cce9e;padding:.3em .8em;display:flex;justify-content:center;align-items:center;height:4em;width:100%}.ex-cards-zone-box .card>.title .name[data-v-bd7eb388]{flex-grow:1;font-size:1.6em}.ex-cards-zone-box .index-box[data-v-bd7eb388]{display:flex;flex-direction:column;margin-top:1em;margin-bottom:1em;margin-left:.8em;gap:1em}.ex-cards-zone-box .index-box .total[data-v-bd7eb388],.ex-cards-zone-box .index-box .percentage[data-v-bd7eb388]{display:flex;flex-wrap:nowrap;align-items:flex-end;gap:.5em}.ex-cards-zone-box .index-box .average-growth[data-v-bd7eb388]{display:flex;flex-wrap:nowrap;align-items:center;gap:.5em}.ex-cards-zone-box .index-box .average-growth .value[data-v-bd7eb388]{color:var(--text-colors, black);border:1px solid var(--text-colors, black);border-radius:1em;padding:.2em .5em;display:flex;flex-wrap:nowrap;align-items:flex-end}.ex-cards-zone-box .index-box .average-growth .value .icon[data-v-bd7eb388]{display:flex;flex-wrap:nowrap;align-items:center;margin-left:.3em;margin-right:.5em;margin-bottom:.19em}.ex-cards-zone-box .index-box .average-growth .value .number[data-v-bd7eb388]{display:flex;flex-wrap:nowrap;align-items:center;margin-right:.5em}
